//
//  ViewController.swift
//  CollectionViewAnimationsDemo
//
//  Created by Jake Marsh on 4/3/17.
//
//

import UIKit
extension UIColor {
    static func random() -> UIColor{
        return UIColor(
            red: CGFloat(drand48()),
            green: CGFloat(drand48()),
            blue: CGFloat(drand48()),
            alpha: 1.0
        )
    }
}

struct Item {
    var color: UIColor
}

class CustomFlowLayout : UICollectionViewFlowLayout {
    var insertingIndexPaths = [IndexPath]()
    
    override func prepare(forCollectionViewUpdates updateItems: [UICollectionViewUpdateItem]) {
        super.prepare(forCollectionViewUpdates: updateItems)
        
        insertingIndexPaths.removeAll()
        
        for update in updateItems {
            if let indexPath = update.indexPathAfterUpdate,
                update.updateAction == .insert {
                insertingIndexPaths.append(indexPath)
            }
        }
    }
    
    override func finalizeCollectionViewUpdates() {
        super.finalizeCollectionViewUpdates()
        
        insertingIndexPaths.removeAll()
    }
    
    override func initialLayoutAttributesForAppearingItem(at itemIndexPath: IndexPath) -> UICollectionViewLayoutAttributes? {
        let attributes = super.initialLayoutAttributesForAppearingItem(at: itemIndexPath)
        
        //if insertingIndexPaths.contains(itemIndexPath) {
        attributes?.alpha = 0.0
        attributes?.transform = CGAffineTransform(scaleX: 0.1, y: 0.1)
        //attributes?.transform = CGAffineTransform(translationX: 0, y: 500.0)
        
        print(attributes)
        //}
        
        return attributes
    }
}

class ViewController: UICollectionViewController {
    var items = [Item]()
    
    @IBAction func add(_ sender: UIBarButtonItem) {
        addItem()
        
        collectionView?.performBatchUpdates({
            self.collectionView?
                .insertItems(at: [IndexPath(item: self.items.count - 1, section: 0)])
        }, completion: nil)
    }
    
    func addItem() {
        items.append(Item(color: .random()))
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        collectionView?.setCollectionViewLayout(CustomFlowLayout(), animated: false)
        
        for _ in 0...10 { addItem() }
    }
    
    override func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return items.count
    }
    
    override func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: "ItemCell", for: indexPath)
        
        cell.contentView.backgroundColor = items[indexPath.item].color
        
        return cell
    }
    
    override func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        items.remove(at: indexPath.item)
        
        collectionView.performBatchUpdates({ 
            collectionView.deleteItems(at: [indexPath])
        }, completion: nil)
    }
}

